import type DemoController from "./control";



// =================================== Control Signals Setup ===================================

export default function setupControlSignals(controller: DemoController) {

    // Control signals consists of a list of time and value pairs.
    // The signal will be smoothly interpolated between them by default.

    // Fade in from black at start of demo
    controller.addControlSignal("fadeFromBlack", [
        [0, 0],
        [1, 0], // A second of black at start
        [7, 1],  // Fade in 
        [145,1],
        [158,0],
    ],
    0.8 // Fade shape
    )




    const snowFlakeSceneStart = 24 //start of scene
    const firstFlakeAwayOffset = 3


    const secondFlakeStart = 59

    controller.addControlSignal("growingSceneOn", [
        [0, 0],
        [snowFlakeSceneStart-2, 0],
        [snowFlakeSceneStart-2, 1],
        [80, 1],
        [80, 0],

    ],    
    0 // Fade shape jump
    )    

    controller.addControlSignal("growingFlakeNumber", [
        [snowFlakeSceneStart + 0, 1], 
        [secondFlakeStart, 1],
        [secondFlakeStart, 2],
        [75, 2],


    ],
    0
    )    
    //WARNING COMMENTS NOT UP TO DATE
    controller.addControlSignal("growingHexSizeScale", [
        [snowFlakeSceneStart + 0, 0], //hex is 0 at start when snowflake start
        [snowFlakeSceneStart + 11, 0], //hex start growing when droplet starts decreasing
        [snowFlakeSceneStart + 12, 1], //hex ready when droplet dissapears
        [snowFlakeSceneStart + 23 + firstFlakeAwayOffset, 1],
        [snowFlakeSceneStart + 26 + firstFlakeAwayOffset, 0], //hex gon again when droplet appeared
        [secondFlakeStart + 0, 0], //hex start growing when droplet starts decreasing
        [secondFlakeStart + 1, 1], //hex ready when droplet dissapears
        [secondFlakeStart + 12, 1],
 //       [secondFlakeStart + 15,0], //hex gon again when droplet appeared
    ],  
    1.0
    )  

    controller.addControlSignal("growingDropletRadiusScale", [
        [snowFlakeSceneStart -2, 1], // water droplet is max from start
        [snowFlakeSceneStart +11 , 1], // to 11 while nothing else happens
        [snowFlakeSceneStart +14, 0], // droplet is gone when snow flake is growing
        [snowFlakeSceneStart +22+firstFlakeAwayOffset, 0], // droplet starts appearing again when flake  dropping
        [snowFlakeSceneStart +25.5+firstFlakeAwayOffset, 1], // droplet back after melting
        [secondFlakeStart +0 , 1], // to 11 while nothing else happens
        [secondFlakeStart +3, 0], // droplet is gone when snow flake is growing
        [secondFlakeStart +11, 0], // droplet starts appearing again when flake  dropping
   //     [secondFlakeStart +14.5, 1], 

    ],    
    1.0 // Fade shape linear
    )

    controller.addControlSignal("growingHalfFlakeScale", [
        [snowFlakeSceneStart + 0, 0], //no brances in the start
        [snowFlakeSceneStart +11, 0], // start when droplet almost gone
        [snowFlakeSceneStart +15, 1], //full before full flake
        [snowFlakeSceneStart +19+ firstFlakeAwayOffset, 1], //start when flake starts decreasing
        [snowFlakeSceneStart +24+firstFlakeAwayOffset, 0], //flake gone
        [secondFlakeStart +0, 0], // start when droplet almost gone
        [secondFlakeStart +4, 1], //full before full flake
        [secondFlakeStart +8, 1], //start when flake starts decreasing
  //      [secondFlakeStart +13, 0], //flake gone

    ],    
    1.0 // Fade shape linear
    )

    controller.addControlSignal("growingFlakeScale", [
        [snowFlakeSceneStart + 0, 0], //no brances in the start
        [snowFlakeSceneStart +11, 0], // start when droplet start
        [snowFlakeSceneStart +17, 1], //full before full flake
        [snowFlakeSceneStart +21.5, 1], //start when flake starts decreasing
        [snowFlakeSceneStart +25, 0], //flake gone
        [secondFlakeStart +0, 0], // start when droplet start
        [secondFlakeStart +6, 1], //full before full flake
        [secondFlakeStart +10.5, 1], //start when flake starts decreasing
     //   [secondFlakeStart +14, 0], //flake gone

    ],    
    1.0 // Fade shape linear
    )

    controller.addControlSignal("growingDropletYOffset", [
        [snowFlakeSceneStart-2, 20],
        [snowFlakeSceneStart-1.9, 12.5],
        [snowFlakeSceneStart + 11, 7],
        [75, 7],
        [80, -10],
    ],    
    0.5 
    )

    controller.addControlSignal("growingFlakerot", [
        [69, 0],
        [69.5, 0.20],
        [70.5, 0.80],
        [71, 0.95],
        [71.4, 1.0],
        [71.4, 0],
        [72, 0.05],
    ],    
    1 // Fade shape linear
    )

    //stuff added to the 
    controller.addControlSignal("growingBaserotmod", [
        [secondFlakeStart-5, 0],
        [secondFlakeStart-1, 0.2],
    ],    
    1 // Fade shape linear
    )

    const dragonTime = 0; 

    //dragon
    controller.addControlSignal("dragonXrotMod", [
        [dragonTime, 1],
        [dragonTime+20, -1],
        [dragonTime+22, -1],
        [dragonTime+40, 1],
    ],    
    1 // Fade shape linear
    )

    const snowstormStart = 80; //has to be larger than 1


    controller.addControlSignal("snowstormStart", [
        [0, 0],
        [snowstormStart, 0],
        [snowstormStart, snowstormStart],

    ],    
    0 // Fade shape jump
    )  

    controller.addControlSignal("stormStartScale", [
        [0, 0],
        [snowstormStart, 0],
        [snowstormStart+2, 1],

    ],    
    1 // Fade shape jump
    )  

    const dragonSceneStart = 100.0
    const dragonRiseTime = 10
    controller.addControlSignal("dragonOnScreen", [
        [0, 0],
        [dragonSceneStart - dragonRiseTime/2, 0], // A second of black at start
        [dragonSceneStart + dragonRiseTime/2, 1],  // Fade in 
    ],
    0.5 // Fade shape
   )

