
import vertCode from '../shaders/mainVertex.glsl'
import mainFrag from '../shaders/mainFragment.glsl'

import * as utils from './utils/utils'

//import Synth from './sound/Synth'
//import Music from './sound/Music'
import DemoController, { ControlSignal } from './control'
import setupControlSignals from './controlSignals'
import Dragon from './Dragon'

import { lumihiutale } from './audio'



//import Synth from './synth/Synth'
//import RetroSong from './synth/RetroSong'


// =================================== Shorthand utilities ===================================

// Shortcut for getting uniform location
const UL: ((u: string) => WebGLUniformLocation | null)  =  (u: string) => gl.getUniformLocation(program, u)
const AL: ((a: string) => GLint)  =  (a: string) => gl.getAttribLocation(program, a)

// =================================== Page Setup ===================================

const b = document.body

// Remove document margin that is automatically added
b.style.margin = "0"

// Currently it seems like it saves quite a few bytes (20) to add this to the start of the html file, instead of doing it with a command like this?
//b.innerHTML = '<canvas id="C"></canvas>'

// Get canvas and it's context.  The canvas is assigned the id C in the html, so it is available with that name as a global variable.
// @ts-ignore
const canvas = (C as HTMLCanvasElement)
const gl: WebGL2RenderingContext  = canvas.getContext("webgl2")!

var frameNumber: number = 0;

// DEBUG: FPS Counter global vars, somehow strip these in prod?  Probably dropped as nothing refers to them
var prevTime: number
var fps: number
var soundTime: number 
var fpsOverlay: HTMLElement
var soundOverlay: HTMLElement
var timeOverlay: HTMLElement

//let synth: Synth
//let music: Music

/*
let synth: Synth
*/

// @ts-ignore    Only runs in development and will be stripped in production builds.
if (process.env.NODE_ENV !== 'production') {
    fps = 60
    soundTime = 0
    prevTime = performance.now()

    performance.mark("sound-start")
    performance.mark("sound-end")
    

    function overlay(yOffs: number, color: string, text: string = ""): HTMLElement {
        var e = document.createElement("div")
        e.innerText = text
        e.style.color = color
        e.style.position = "absolute"
        e.style.left = "10px"
        e.style.top = yOffs + "px"
        e.style.zIndex = "1"
        e.style.fontFamily = "sans"
        e.style.fontWeight = "bold"
        e.style.fontSize = "20px"
        e.style.textShadow = "1px 1px 1px black, 0px 0px 10px white"
        b.appendChild(e)
        return e
    }

    // FPS Counter
    fpsOverlay = overlay(10, "black", "")

    // Sound time milliseconds
    soundOverlay = overlay(40, "#09e")

    // Time
    timeOverlay = overlay(70, "lightgrey")


}

// =================================== Control Signals Setup ===================================

const controller = new DemoController()

setupControlSignals(controller)


// =================================== Dragon  Setup ===================================

const dragon: Dragon = new Dragon()


// =================================== Shader Setup ===================================

// Create Program
const program = createProgram()
gl.useProgram(program)

// Create vertex buffer (2 triangles covering whole viewport)
const buffer = gl.createBuffer()
gl.bindBuffer(0x8892, buffer) // gl.ARRAY_BUFFER
gl.bufferData(0x8892, //gl.ARRAY_BUFFER
    new Float32Array([
        -1.0, -1.0,
         1.0, -1.0,
        -1.0,  1.0,
         1.0, -1.0,
         1.0,  1.0,
        -1.0,  1.0
    ]),
    0x88E4 // gl.STATIC_DRAW
)

const vertPos = AL('p')
gl.enableVertexAttribArray(vertPos)
gl.vertexAttribPointer(vertPos, 2, 0x1406, false, 0, 0) // gl.FLOAT

// =================================== Texture buffer setup ===================================
/*
const cloudBuffer = gl.createFramebuffer();
const cloudTex = gl.createTexture();
*/
// =================================== Setup resize listening ===================================

// Initialize size and listen to resizes
var w: number
var h: number
ucs()
window.onresize = ucs
addEventListener('resize', ucs)

// Update canvas size
function ucs() {
    w = canvas.width = innerWidth
    h = canvas.height = innerHeight
    //gl.viewport(0, 0, w, h)

    // Update texture size
    //gl.deleteTexture(cloudTex)
    /*
    gl.bindTexture(gl.TEXTURE_2D, cloudTex);
    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, w, h, 0, gl.RGBA, gl.UNSIGNED_BYTE, null);

    gl.bindFramebuffer(gl.FRAMEBUFFER, cloudBuffer);
    gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, cloudTex, 0);

    gl.bindBuffer(gl.FRAMEBUFFER, cloudBuffer);
    */

    gl.viewport(0, 0, w, h);    
}

// =================================== Start Button ===================================
// Need interactive click to allow sound

let startTime = performance.now()

let stopDemo = false


// light
// Salivalot
//const SalivalotEnabled = true
//if (SalivalotEnabled || process.env.NODE_ENV === 'production') {
    let socket = new WebSocket("ws://valot.instanssi:9910");
    socket.onopen = function (event) {
        var data = new Uint8Array(166);
        data[0] = 1;
        data[1] = 0;  
        data[2] = 97;
        data[3] = 0; 
        function lightingLoop() {
            let time = (performance.now() - startTime) / 1000.0 +150
            if (time <  24){ //fade in
                for(var i = 1; i < 23; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = (time/23)*55; // Punainen//these need time relevant parameters
                    data[p + 4] = (time/23)*25; // Vihreä
                    data[p + 5] = (time/23)*80; // Sininen
        
                }
            }
            else if (time <= 35){
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 55-((time-23)*2); // Punainen//these need time relevant parameters
                    data[p + 4] = 25+((time-23)); // Vihreä
                    data[p + 5] = 80+((time-23)*2); // Sininen
        
                }
            }
            else if (time <= 43){
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 31+((time-35)*10); // Punainen//these need time relevant parameters
                    data[p + 4] = 37+((time-35)*12); // Vihreä
                    data[p + 5] = 104+((time-35)*2); // Sininen
        
                }
            }
            else if (time <= 58){
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 111-((time-43)*6); // Punainen//these need time relevant parameters
                    data[p + 4] = 133-((time-43)*7); // Vihreä
                    data[p + 5] = 120-((time-43)*2); // Sininen
        
                }
            }
            else if (time <= 68){
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 21 + ((time-58)*15); // Punainen//these need time relevant parameters
                    data[p + 4] = 28 +((time-58)*20); // Vihreä
                    data[p + 5] = 90 + ((time-58)*10); // Sininen
        
                }
            }
            else if (time <=  71.5){
                for(var i = 1; i < 24; i+=2) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 171- ((time-68)*45); // Punainen//these need time relevant parameters
                    data[p + 4] = 228-((time-68)*65)
                    data[p + 5] = 190-((time-68)*40); // Sininen
        
                } 
            }

            else if (time <= 80){
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 13.5 + ((time-71.5)*5); // Punainen//these need time relevant parameters
                    data[p + 4] = 0.5 +((time-71.5)*20)
                    data[p + 5] = 50+ ((time-71.5)*20); // Sininen
        
                }
            }
            else if (time <= 145){
                var p = 4 + 6 * i; 
                data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                data[p + 3] = 160; // Punainen//these need time relevant parameters
                data[p + 4] = 180; // Vihreä
                data[p + 5] = 220; // Sininen
            }
            else if (time <= 160){
                var p = 4 + 6 * i; 
                data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                data[p + 3] = 160-(160*(time-145)/(158-145)); // Punainen//these need time relevant parameters
                data[p + 4] = 180-(180*(time-145)/(158-145));  // Vihreä
                data[p + 5] = 220-(220*(time-145)/(158-145));  // Sininen
            }

            else{
                for(var i = 1; i < 24; i++) {

                    var p = 4 + 6 * i; 
                    data[p + 0] = 1; // Tehosteen tyyppi on yksi eli valo
                    data[p + 1] = i; // Ensimmäinen valo löytyy indeksistä nolla
                    data[p + 3] = 0; // Punainen//these need time relevant parameters
                    data[p + 4] = 0; // Vihreä
                    data[p + 5] = 0; // Sininen
        
                }

            }

            socket.send(data); 
            setTimeout(lightingLoop, 10);
        }
        lightingLoop();
    }
//}



const sb = document.createElement("button")
sb.style.fontSize = "20vh"
sb.style.position = "absolute"
sb.style.left = "40%"
sb.style.top = "50%"
sb.style.width = "40vw"
sb.style.height = "30vh"
sb.style.borderRadius = "5vh"
sb.innerText="Start"
b.appendChild(sb)
b.style.background = "#000"
const audio = new Audio(lumihiutale)
let start = () => {
    // Start time
    startTime = performance.now()

    // Hide mouse cursor
    b.style.cursor = "none"

    // Hide start button
    sb.hidden=true

    // @ts-ignore    Only go fullscreen in production, it can be a bit inconvinient in dev.
    if (process.env.NODE_ENV === 'production') {
        // Go fullscreen
        utils.requestFullScreen(document.body);
    }

    //synth = new Synth()
    //music = new Music(synth)
    audio.currentTime = 0
    audio.play()

    // Detect keys
    const weAreInProd = process.env.NODE_ENV === 'production'
    document.body.onkeydown = (e: Event) => {
        let handled = false;
        if (e.key === "Escape") {
            // Stop demo
            document.body.innerHTML = "" // Clear page
            stopDemo = true // Stop animation loop
            //synth.stop() // Stop music
            audio.pause()
        } 
        else if (e.key === " ") {
            
                // Reset time
                startTime = performance.now()

                // Reset dragon
                dragon.reset()

                // Restart music from 0
                //synth.restart()
                audio.currentTime = 0
                audio.play()
            
        }
    }

    // Start animation loop
    frame()
}
sb.onclick = start
 
// @ts-ignore
if (process.env.NODE_ENV !== 'production') {
    // In development environment, autostart without button
//    start()
}



// =================================== Animation loop ===================================

// Animation loop
function frame () {

    if (!stopDemo) {
        render()
    
        requestAnimationFrame(frame)



    }


}

function render() {
/*
    // Set clear color to black, fully opaque
    gl.clearColor(0.0, 0.0, 0.0, 1.0);
    // Clear the color buffer with specified clear color
    gl.clear(gl.COLOR_BUFFER_BIT);
*/


    // Set values to program variables
    let time = (performance.now() - startTime) / 1000
    gl.uniform1f(UL('time'), time) // Time in seconds since start
    gl.uniform2f(UL('screenSize'), w, h) // Screen size in pixels
    gl.uniform1ui(UL('frameNum'), frameNumber) // Number of frames since start

    controller.update(time)
    controller.updateShaderParameters(gl, program)

    dragon.updateShaderParameters(time, gl, program)
    

    // Render geometry
    gl.clear( 0x00004000 | 0x00000100) // gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT
    gl.drawArrays(0x0004, 0, 6) // gl.TRIANGLES

    

    // @ts-ignore    Only runs in development and will be stripped in production builds.
    if (process.env.NODE_ENV !== 'production') {
        // FPS counter
        const n = performance.now()
        var dt = 0.001 * (n - prevTime)
        prevTime = n
        const fpsDisplayTimeSmoothing = 0.9
        if (dt > 0) fps = fpsDisplayTimeSmoothing *fps + (1.0-fpsDisplayTimeSmoothing) * 1/dt
        fpsOverlay.innerText = "FPS: " + Math.round(fps)
        fpsOverlay.style.color = fps >= 55 ? "#006600" : "#882200"

        // Sound time
        let m = performance.measure("sound-time", "sound-start", "sound-end")
        const soundDisplayTimeSmoothing = 0.95
        soundTime = soundDisplayTimeSmoothing * soundTime + (1.0 - soundDisplayTimeSmoothing) * m.duration
        soundOverlay.innerText = "Sound: " + Math.round(soundTime*100)/100 + " ms"

        // Time
        timeOverlay.innerText =  "Frame: " + frameNumber.toString().padStart(5, '0') + " Time: " + time + " s"
    }

    
    
    frameNumber++;
    frameNumber %= 100000; // We'd like this to stay an integer.  
    

}


// =================================== Shader Utils ===================================

// Create webgl shader program from shaders
function createProgram() {
    var fragmentShader = mainFrag

    // @ts-ignore    Only runs in development and will be stripped in production builds.
    if (process.env.NODE_ENV !== 'production') {
        // Make it easier to find errors at specific line numbers
        console.log("Fragment shader code:\n" + fragmentShader.split("\n").map((line, index) => "" + (index + 1) + "| " + line).join('\n'))
    }


    const program = gl.createProgram()!
    var vertShader = createShader(vertCode, 0x8B31) // gl.VERTEX_SHADER
    var fragShader = createShader(fragmentShader, 0x8B30) // gl.FRAGMENT_SHADER

    gl.attachShader(program, vertShader)
    gl.attachShader(program, fragShader)

    //gl.deleteShader(vertShader)
    //gl.deleteShader(fragShader)

    gl.linkProgram(program)

    // @ts-ignore    Only runs in development and will be stripped in production builds.
    if (process.env.NODE_ENV !== 'production') {
        if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
            throw "WebGL program error:\n" +
                  "VALIDATE_STATUS: " + gl.getProgramParameter(program, gl.VALIDATE_STATUS) + "\n" +
                  "ERROR: " + gl.getError()
        }
    }    

    return program;
}


// Create a shader from source
function createShader(source: string, type: GLenum): WebGLShader {
    var shader = gl.createShader(type)!

    gl.shaderSource(shader, source)
    gl.compileShader(shader);

    // @ts-ignore    Only runs in development and will be stripped in production builds.
    if (process.env.NODE_ENV !== 'production') {
        if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
            throw "WebGL shader error:\n" +
            (type == gl.VERTEX_SHADER ? "VERTEX" : "FRAGMENT") + " SHADER:\n" +
            gl.getShaderInfoLog(shader)
        }
    }

    return shader
}